@@ -23,7 +23,7 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ident "$Id: depmod.c 1.2 Tue, 09 Jan 2001 01:27:37 +1100 kaos $"
+#ident "$Id: depmod.c 1.3 Tue, 23 Jan 2001 05:13:16 +1100 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -96,6 +96,8 @@ struct isapnp_card_id {
 /* Version 1 table extracted from 2.4.0-test10-pre3 + David Brownell
  * usb-device_id patch 2.
  * Version 2 table extracted from 2.4.0 + Keith Owens 2.4.0-depmod patch.
+ * Update 23/01/2001.  Drop version 2 and old version 1.  New version 1
+ * is now from kernel 2.4.0.
  */
 
 #ifndef __u16
@@ -121,11 +123,23 @@ struct isapnp_card_id {
  * of the structure.  KAO moved it to before driver_info and defined that format
  * as a version 2 table for kernel 2.4.1 and modutils 2.4.1.  USB modules will
  * not be supported in standard kernel 2.4.0, with any version of modutils.
+ *
+ * Update 23/01/2001.  USB maintainers will not support anybody using the USB
+ * table format in 2.4.0-prerelease or earlier.  Without their support, Linus
+ * will not apply my patch to add version numbers to kernel tables.  Without the
+ * version number, depmod cannot distinguish between 2.4.0-prelease or earlier
+ * formats and 2.4.0.  So it has to assume 2.4.0 format for everybody.  This
+ * will break USB for anybody using modutils >= 2.4.2 on kernels before 2.4.0.
  */
 
 unsigned int usb_device_id_ver;
 
 struct usb_device_id {
+	/* This bitmask is used to determine which of the following fields
+	 * are to be used for matching.
+	 */
+	__u16		match_flags;
+
 	/*
 	 * vendor/product codes are checked, if vendor is nonzero
 	 * Range is for device revision (bcdDevice), inclusive;
@@ -151,15 +165,10 @@ struct usb_device_id {
 	__u8		bInterfaceSubClass;
 	__u8		bInterfaceProtocol;
 
-	/* This bitmask is used to determine which of the preceding fields
-	 * are to be used for matching.
-	 */
-	__u16		match_flags;	/* New in version 2 */
-
 	/*
 	 * for driver's use; not involved in driver matching.
 	 */
-	unsigned tgt_long		driver_info;
+	unsigned long	driver_info;
 };
 
 /* Supplied by Adam J. Richter, approx 2.4.0-test13, modified by KAO.
@@ -415,7 +424,8 @@ static void extract_generic_string(struct obj_file *f, MODULE *mod)
  *	Note: assumes same machine and arch for depmod and module.
  */
 static void extract_version(struct obj_file *f, void *image, unsigned long m_size,
-			    const char *table, unsigned int *version, int max_version)
+			    const char *table, unsigned int *version,
+			    int min_version, int max_version)
 {
 	ElfW(Addr) ref_ver;
 	unsigned tgt_long ver;
@@ -424,20 +434,20 @@ static void extract_version(struct obj_file *f, void *image, unsigned long m_siz
 	strcat(name, table);
 	strcat(name, "_ver");
 	ref_ver = obj_symbol_final_value(f, obj_find_symbol(f, name));
-	/* Kernels before 2.4.1 do not have __module_<table>_ver, default to version 1 */
+	/* Kernels before 2.4.1 do not have __module_<table>_ver, default to min_version */
 	if (ref_ver) {
 		if (!in_range(f, m_size, ref_ver, sizeof(ver)))
 			return;
 		memcpy(&ver, (char *)image + ref_ver - f->baseaddr, sizeof(ver));
 	}
 	else
-		ver = 1;
+		ver = min_version;
 	if (*version && *version != ver) {
 		error("Modules have a mixture of version %d and version %" tgt_long_fmt "d %s tables",
 			*version, ver, table);
 		exit(-1);
 	}
-	if (ver > max_version) {
+	if (ver < min_version || ver > max_version) {
 		error("Module %s has an unknown version (%" tgt_long_fmt "d) %s table%s",
 			f->filename, ver, table, has_kernel_changed);
 		exit(-1);
@@ -468,7 +478,7 @@ static void extract_pci_device_id(struct obj_file *f, void *image, unsigned long
 	if (!in_range(f, m_size, ref_ref_pci, sizeof(ref_pci)))
 		return;
 	memcpy(&ref_pci, (char *)image + ref_ref_pci - f->baseaddr, sizeof(ref_pci));
-	extract_version(f, image, m_size, "pci_device", &pci_device_id_ver, 1);
+	extract_version(f, image, m_size, "pci_device", &pci_device_id_ver, 1, 1);
 	if (pci_device_size != sizeof(pci_device)) {
 		error("Unexpected value (%" tgt_long_fmt "d) in '%s' for pci_device_size%s",
 			pci_device_size, f->filename, has_kernel_changed);
@@ -503,7 +513,7 @@ static void extract_isapnp_device_id(struct obj_file *f, void *image, unsigned l
 	if (!in_range(f, m_size, ref_ref_isapnp, sizeof(ref_isapnp)))
 		return;
 	memcpy(&ref_isapnp, (char *)image + ref_ref_isapnp - f->baseaddr, sizeof(ref_isapnp));
-	extract_version(f, image, m_size, "isapnp_device", &isapnp_device_id_ver, 1);
+	extract_version(f, image, m_size, "isapnp_device", &isapnp_device_id_ver, 1, 1);
 	if (isapnp_device_size != sizeof(isapnp_device)) {
 		error("Unexpected value (%" tgt_long_fmt "d) in '%s' for isapnp_device_size%s",
 			isapnp_device_size, f->filename, has_kernel_changed);
@@ -572,7 +582,7 @@ static void extract_usb_device_id(struct obj_file *f, void *image, unsigned long
 	if (!in_range(f, m_size, ref_ref_usb, sizeof(ref_usb)))
 		return;
 	memcpy(&ref_usb, (char *)image + ref_ref_usb - f->baseaddr, sizeof(ref_usb));
-	extract_version(f, image, m_size, "usb_device", &usb_device_id_ver, 2);
+	extract_version(f, image, m_size, "usb_device", &usb_device_id_ver, 1, 1);
 	if (usb_device_size != sizeof(usb_device)) {
 		error("Unexpected value (%" tgt_long_fmt "d) in '%s' for usb_device_size%s",
 		usb_device_size, f->filename, has_kernel_changed);
@@ -608,7 +618,7 @@ static void extract_parport_device_id(struct obj_file *f, void *image, unsigned
 	if (!in_range(f, m_size, ref_ref_parport, sizeof(ref_parport)))
 		return;
 	memcpy(&ref_parport, (char *)image + ref_ref_parport - f->baseaddr, sizeof(ref_parport));
-	extract_version(f, image, m_size, "parport_device", &parport_device_id_ver, 1);
+	extract_version(f, image, m_size, "parport_device", &parport_device_id_ver, 1, 1);
 	if (parport_device_size != sizeof(parport_device)) {
 		error("Unexpected value (%" tgt_long_fmt "d) in '%s' for parport_device_size%s",
 			parport_device_size, f->filename, has_kernel_changed);
@@ -1150,8 +1160,8 @@ static void prtdepend(char *base_dir, int nflag)
 
 	ptmod = modules;
 	fprintf(usbmap, "# usb module         ");
-	if (usb_device_id_ver > 1)
-		fprintf(usbmap, "match_flags ");
+	/* Requires all users to be on kernel 2.4.0 or later */
+	fprintf(usbmap, "match_flags ");
 	fprintf(usbmap, "idVendor idProduct bcdDevice_lo bcdDevice_hi"
 		" bDeviceClass bDeviceSubClass bDeviceProtocol bInterfaceClass bInterfaceSubClass"
 		" bInterfaceProtocol driver_info\n");
@@ -1162,10 +1172,9 @@ static void prtdepend(char *base_dir, int nflag)
 			continue;
 		for (j = 0; j < ptmod->n_usb_device; j++, usb_device++) {
 			fprintf(usbmap, "%-20s ", shortname(ptmod->name));
-			if (usb_device_id_ver > 1) {
-				fprintf(usbmap, "0x%0*x      ",
+			/* Requires all users to be on kernel 2.4.0 or later */
+			fprintf(usbmap, "0x%0*x      ",
 				(int)(2*sizeof(usb_device->match_flags)), usb_device->match_flags);
-			}
 			fprintf(usbmap, "0x%0*x   0x%0*x    0x%0*x       0x%0*x       "
 				"0x%0*x         0x%0*x            0x%0*x            0x%0*x            0x%0*x               "
 				"0x%0*x               0x%0*" tgt_long_fmt "x\n",
