@@ -33,7 +33,7 @@
  *   Keith Owens <kaos@ocs.com.au> November 2000.
  */
 
-#ident "$Id: modinfo.c 1.1 Fri, 05 Jan 2001 12:45:19 +1100 kaos $"
+#ident "$Id: modinfo.c 1.3 Sun, 25 Mar 2001 21:13:48 +1000 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -97,7 +97,7 @@ static int convert_escape (int c)
 
   return(c);
 }
- 
+
 
 static const char *get_modinfo_value(struct obj_file *f, const char *key)
 {
@@ -163,7 +163,7 @@ static char *format_query_string(struct obj_file *f, const char *format)
 
 	while (*in_str && out_str < last_char) {
 		/* Just copy normal characters into the output. */
-		if (*in_str != '%') 
+		if (*in_str != '%')
 		  {
 		    if (*in_str == '\\')
 		      c = convert_escape(*++in_str);
@@ -385,7 +385,8 @@ int SHOW_MODULE_INFO(const char *filename, const char *fmtstr, int do_parameters
 
 	if (do_parameters) {
 		show_module_parameters(f);
-	} else if (fmtstr) {
+	}
+	if (fmtstr) {
 		p = format_query_string(f, fmtstr);
 		if (p)
 			fputs(p, stdout);
@@ -410,7 +411,7 @@ extern void modinfo_usage(void);	/* Use the copy in the 64 bit version */
 #else	/* Common 64 bit version or any non common code - compile usage routine */
 void modinfo_usage(void)
 {
-	printf("usage: modinfo <parameters> <module>\n");
+	printf("usage: modinfo <options> <module>\n");
 	printf("  -a, --author            display module author\n");
 	printf("  -d, --description       display module description\n");
 	printf("  -n, --filename          display module filename\n");
@@ -440,22 +441,26 @@ int main(int argc, char *argv[])
 		{"help", 0, 0, 'h'},
 		{0, 0, 0, 0}
 	};
-	int do_parameters = 0, opt;
-	char *fmtstr = NULL;
+	int do_parameters = 1, opt;
+	char *fmtstr = "filename    - %n\ndescription - %d\nauthor      - %a\n";
 
 	while ((opt = getopt_long(argc, argv, "adnq:pf:Vh", long_opts, NULL)) != EOF)
 		switch (opt) {
 		case 'a':
 			fmtstr = "%a\n";
+			do_parameters = 0;
 			break;
 		case 'd':
 			fmtstr = "%d\n";
+			do_parameters = 0;
 			break;
 		case 'n':
 			fmtstr = "%n\n";
+			do_parameters = 0;
 			break;
 		case 'f':
 			fmtstr = xstrdup(optarg);
+			do_parameters = 0;
 			break;
 		case 'p':
 			do_parameters = 1;
